@@ -1,24 +1,11 @@
 /*
- *   Copyright (C) 2018 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *   SPDX-FileCopyrightText: 2018 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library/Lesser General Public License
- *   version 2, or (at your option) any later version, as published by the
- *   Free Software Foundation
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library/Lesser General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *   SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "ReadFile.h"
-#include <QDebug>
+#include "discover_debug.h"
 
 ReadFile::ReadFile()
 {
@@ -101,7 +88,7 @@ void ReadFile::setFilter(const QString& filter)
 {
     m_filter = QRegularExpression(filter);
     if (!m_filter.isValid())
-        qDebug() << "error" << m_filter.errorString();
+        qCDebug(DISCOVER_LOG) << "error" << m_filter.errorString();
     Q_ASSERT(filter.isEmpty() || m_filter.isValid());
 }
 
