@@ -12,7 +12,6 @@
 #include <QDebug>
 #include "utils.h"
 #include <KLocalizedString>
-#include <AppStreamQt/component.h>
 #include <AppStreamQt/release.h>
 #include <AppStreamQt/screenshot.h>
 #if APPSTREAM_HAS_SPDX
@@ -97,11 +96,7 @@ QStringList AppStreamUtils::appstreamIds(const QUrl &appstreamUrl)
     ret += appstreamUrl.host().isEmpty() ? appstreamUrl.path() : appstreamUrl.host();
     if (appstreamUrl.hasQuery()) {
         QUrlQuery query(appstreamUrl);
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-        ret << query.queryItemValue(QStringLiteral("alt")).split(QLatin1Char(','), QString::SkipEmptyParts);
-#else
         ret << query.queryItemValue(QStringLiteral("alt")).split(QLatin1Char(','), Qt::SkipEmptyParts);
-#endif
     }
     return ret;
 }
