@@ -1,27 +1,12 @@
-/***************************************************************************
- *   Copyright © 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>       *
- *   Copyright © 2017 Jan Grulich <jgrulich@redhat.com>                    *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- ***************************************************************************/
+/*
+ *   SPDX-FileCopyrightText: 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *   SPDX-FileCopyrightText: 2017 Jan Grulich <jgrulich@redhat.com>
+ *
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+ */
 
 #include "FlatpakBackend.h"
 #include "FlatpakFetchDataJob.h"
-#include "FlatpakResource.h"
 #include "FlatpakSourcesBackend.h"
 #include "FlatpakJobTransaction.h"
 
@@ -34,7 +19,6 @@
 #include <appstream/AppStreamUtils.h>
 
 #include <AppStreamQt/bundle.h>
-#include <AppStreamQt/component.h>
 #include <AppStreamQt/icon.h>
 #include <AppStreamQt/metadata.h>
 
@@ -124,10 +108,12 @@ FlatpakBackend::FlatpakBackend(QObject* parent)
 FlatpakBackend::~FlatpakBackend()
 {
     g_cancellable_cancel(m_cancellable);
-    m_threadPool.waitForDone(200);
-    m_threadPool.clear();
     for(auto inst : qAsConst(m_installations))
         g_object_unref(inst);
+    if (!m_threadPool.waitForDone(200)) {
+        qDebug() << "could not kill them all" << m_threadPool.activeThreadCount();
+    }
+    m_threadPool.clear();
 
     g_object_unref(m_cancellable);
 }
@@ -223,23 +209,19 @@ FlatpakRemote * FlatpakBackend::getFlatpakRemoteByUrl(const QString &url, Flatpa
     return nullptr;
 }
 
-FlatpakInstalledRef * FlatpakBackend::getInstalledRefForApp(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource) const
+FlatpakInstalledRef * FlatpakBackend::getInstalledRefForApp(FlatpakResource *resource) const
 {
-    FlatpakInstalledRef *ref = nullptr;
     g_autoptr(GError) localError = nullptr;
 
-    if (!flatpakInstallation) {
-        return ref;
-    }
-
     const auto type = resource->resourceType() == FlatpakResource::DesktopApp ? FLATPAK_REF_KIND_APP : FLATPAK_REF_KIND_RUNTIME;
 
-    return flatpak_installation_get_installed_ref(flatpakInstallation,
+    FlatpakInstalledRef *ref = flatpak_installation_get_installed_ref(resource->installation(),
                                                  type,
                                                  resource->flatpakName().toUtf8().constData(),
                                                  resource->arch().toUtf8().constData(),
                                                  resource->branch().toUtf8().constData(),
                                                  m_cancellable, &localError);
+    return ref;
 }
 
 FlatpakResource * FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *flatpakInstallation, FlatpakInstalledRef *ref) const
@@ -274,7 +256,7 @@ FlatpakResource * FlatpakBackend::getRuntimeForApp(FlatpakResource *resource) co
 
     // TODO if runtime wasn't found, create a new one from available info
     if (!runtime) {
-        qWarning() << "could not find runtime" << runtimeInfo << resource;
+        qWarning() << "could not find runtime" << runtimeName << resource;
     }
 
     return runtime;
@@ -296,7 +278,11 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url)
         return nullptr;
     }
 
+
+    gsize len = 0;
     g_autoptr(GBytes) metadata = flatpak_bundle_ref_get_metadata(bundleRef);
+    const QByteArray metadataContent((char *)g_bytes_get_data(metadata, &len));
+
     appstreamGz = flatpak_bundle_ref_get_appstream(bundleRef);
     if (appstreamGz) {
         g_autoptr(GZlibDecompressor) decompressor = nullptr;
@@ -347,8 +333,6 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url)
             return nullptr;
         }
 
-        gsize len = 0;
-        QByteArray metadataContent = QByteArray((char *)g_bytes_get_data(metadata, &len));
         tempFile.write(metadataContent);
         tempFile.close();
 
@@ -362,9 +346,6 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakBundle(const QUrl &url)
     }
 
     FlatpakResource *resource = new FlatpakResource(asComponent, preferredInstallation(), this);
-
-    gsize len = 0;
-    QByteArray metadataContent = QByteArray((char *)g_bytes_get_data(metadata, &len));
     if (!updateAppMetadata(resource, metadataContent)) {
         delete resource;
         qWarning() << "Failed to update metadata from app bundle";
@@ -466,14 +447,13 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
 
     QUrl runtimeUrl = QUrl(settings.value(QStringLiteral("Flatpak Ref/RuntimeRepo")).toString());
     if (!runtimeUrl.isEmpty()) {
-        auto installation = preferredInstallation();
         // We need to fetch metadata to find information about required runtime
         auto fw = new QFutureWatcher<QByteArray>(this);
-        connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, installation, resource, fw, runtimeUrl]() {
+        connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, resource, fw, runtimeUrl]() {
             const auto metadata = fw->result();
             // Even when we failed to fetch information about runtime we still want to show the application
             if (metadata.isEmpty()) {
-                onFetchMetadataFinished(installation, resource, metadata);
+                Q_EMIT onFetchMetadataFinished(resource, metadata);
             } else {
                 updateAppMetadata(resource, metadata);
 
@@ -494,7 +474,7 @@ FlatpakResource * FlatpakBackend::addAppFromFlatpakRef(const QUrl &url)
             }
             fw->deleteLater();
         });
-        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, installation, resource));
+        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, resource, m_cancellable));
     } else {
         addResource(resource);
     }
@@ -559,11 +539,7 @@ void FlatpakBackend::addResource(FlatpakResource *resource)
         qWarning() << "Failed to parse metadata from app bundle for" << resource->name();
     }
 
-    auto installation = resource->installation();
-    updateAppState(installation, resource);
-
-    // This will update also metadata (required runtime)
-    updateAppSize(installation, resource);
+    updateAppState(resource);
 
     m_resources.insert(resource->uniqueId(), resource);
     if (!resource->extends().isEmpty()) {
@@ -589,7 +565,8 @@ public:
             qWarning() << "No appstream dir for" << flatpak_remote_get_name(m_remote);
             return {};
         }
-        return QString::fromUtf8(g_file_get_path(appstreamDir));
+        g_autofree char *path_str = g_file_get_path(appstreamDir);
+        return QString::fromUtf8(path_str);
     }
 
     QString name() const
@@ -618,7 +595,7 @@ bool FlatpakBackend::loadAppsFromAppstreamData(FlatpakInstallation *flatpakInsta
 {
     Q_ASSERT(flatpakInstallation);
 
-    GPtrArray *remotes = flatpak_installation_list_remotes(flatpakInstallation, m_cancellable, nullptr);
+    g_autoptr(GPtrArray) remotes = flatpak_installation_list_remotes(flatpakInstallation, m_cancellable, nullptr);
     if (!remotes) {
         return false;
     }
@@ -629,7 +606,8 @@ bool FlatpakBackend::loadAppsFromAppstreamData(FlatpakInstallation *flatpakInsta
         FlatpakRemote *remote = FLATPAK_REMOTE(g_ptr_array_index(remotes, i));
         g_autoptr(GFile) fileTimestamp = flatpak_remote_get_appstream_timestamp(remote, flatpak_get_default_arch());
 
-        QFileInfo fileInfo = QFileInfo(QString::fromUtf8(g_file_get_path(fileTimestamp)));
+        g_autofree char *path_str = g_file_get_path(fileTimestamp);
+        QFileInfo fileInfo = QFileInfo(QString::fromUtf8(path_str));
         // Refresh appstream metadata in case they have never been refreshed or the cache is older than 6 hours
         if (!fileInfo.exists() || fileInfo.lastModified().toUTC().secsTo(QDateTime::currentDateTimeUtc()) > 21600) {
             refreshAppstreamMetadata(flatpakInstallation, remote);
@@ -672,10 +650,18 @@ void FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, F
     const auto sourceName = source.name();
     connect(fw, &QFutureWatcher<QList<AppStream::Component>>::finished, this, [this, fw, flatpakInstallation, appstreamIconsPath, sourceName]() {
         const auto components = fw->result();
-        foreach (const AppStream::Component& appstreamComponent, components) {
+        QVector<FlatpakResource*> resources;
+        for (const AppStream::Component& appstreamComponent : components) {
             FlatpakResource *resource = new FlatpakResource(appstreamComponent, flatpakInstallation, this);
             resource->setIconPath(appstreamIconsPath);
             resource->setOrigin(sourceName);
+            if (resource->resourceType() == FlatpakResource::Runtime) {
+                resources.prepend(resource);
+            } else {
+                resources.append(resource);
+            }
+        }
+        for (auto resource : qAsConst(resources)) {
             addResource(resource);
         }
 
@@ -721,6 +707,7 @@ bool FlatpakBackend::loadInstalledApps(FlatpakInstallation *flatpakInstallation)
     const QString pathExports = FlatpakResource::installationPath(flatpakInstallation) + QLatin1String("/exports/");
     const QString pathApps = pathExports + QLatin1String("share/applications/");
 
+    QVector<FlatpakResource*> resources;
     for (uint i = 0; i < refs->len; i++) {
         FlatpakInstalledRef *ref = FLATPAK_INSTALLED_REF(g_ptr_array_index(refs, i));
 
@@ -744,7 +731,7 @@ bool FlatpakBackend::loadInstalledApps(FlatpakInstallation *flatpakInstallation)
 
             cid.setId(QString::fromLatin1(flatpak_ref_get_name(FLATPAK_REF(ref))));
 #if FLATPAK_CHECK_VERSION(1,1,2)
-            cid.setName(QString::fromLatin1(flatpak_installed_ref_get_appdata_name(ref)));
+            cid.setName(QString::fromUtf8(flatpak_installed_ref_get_appdata_name(ref)));
 #endif
         } else
             cid = metadata.component();
@@ -756,8 +743,14 @@ bool FlatpakBackend::loadInstalledApps(FlatpakInstallation *flatpakInstallation)
         resource->setOrigin(QString::fromUtf8(flatpak_installed_ref_get_origin(ref)));
         resource->updateFromRef(FLATPAK_REF(ref));
 
-        addResource(resource);
+        if (resource->resourceType() == FlatpakResource::Runtime) {
+            resources.prepend(resource);
+        } else {
+            resources.append(resource);
+        }
     }
+    for (auto resource : qAsConst(resources))
+        addResource(resource);
 
     return true;
 }
@@ -765,9 +758,7 @@ bool FlatpakBackend::loadInstalledApps(FlatpakInstallation *flatpakInstallation)
 void FlatpakBackend::loadLocalUpdates(FlatpakInstallation *flatpakInstallation)
 {
     g_autoptr(GError) localError = nullptr;
-    g_autoptr(GPtrArray) refs = nullptr;
-
-    refs = flatpak_installation_list_installed_refs(flatpakInstallation, m_cancellable, &localError);
+    g_autoptr(GPtrArray) refs = flatpak_installation_list_installed_refs(flatpakInstallation, m_cancellable, &localError);
     if (!refs) {
         qWarning() << "Failed to get list of installed refs for listing updates:" << localError->message;
         return;
@@ -791,7 +782,7 @@ void FlatpakBackend::loadLocalUpdates(FlatpakInstallation *flatpakInstallation)
         FlatpakResource *resource = getAppForInstalledRef(flatpakInstallation, ref);
         if (resource) {
             resource->setState(AbstractResource::Upgradeable);
-            updateAppSize(flatpakInstallation, resource);
+            updateAppSize(resource);
         }
     }
 }
@@ -808,6 +799,10 @@ void FlatpakBackend::loadRemoteUpdates(FlatpakInstallation* installation)
     acquireFetching(true);
     fw->setFuture(QtConcurrent::run(&m_threadPool, [installation, this]() -> GPtrArray * {
         g_autoptr(GError) localError = nullptr;
+        if (g_cancellable_is_cancelled(m_cancellable)) {
+            qWarning() << "don't issue commands after cancelling";
+            return {};
+        }
         GPtrArray *refs = flatpak_installation_list_installed_refs_for_update(installation, m_cancellable, &localError);
         if (!refs) {
             qWarning() << "Failed to get list of installed refs for listing updates: " << localError->message;
@@ -828,7 +823,7 @@ void FlatpakBackend::onFetchUpdatesFinished(FlatpakInstallation *flatpakInstalla
         FlatpakResource *resource = getAppForInstalledRef(flatpakInstallation, ref);
         if (resource) {
             resource->setState(AbstractResource::Upgradeable);
-            updateAppSize(flatpakInstallation, resource);
+            updateAppSize(resource);
         } else
             qWarning() << "could not find updated resource" << flatpak_ref_get_name(FLATPAK_REF(ref)) << m_resources.size();
     }
@@ -864,11 +859,13 @@ public:
         , m_installation(installation)
         , m_remote(remote)
     {
+        g_object_ref(m_remote);
         connect(this, &FlatpakRefreshAppstreamMetadataJob::finished, this, &QObject::deleteLater);
     }
 
     ~FlatpakRefreshAppstreamMetadataJob()
     {
+        g_object_unref(m_remote);
         g_object_unref(m_cancellable);
     }
 
@@ -928,12 +925,14 @@ bool FlatpakBackend::setupFlatpakInstallations(GError **error)
         return true;
     }
 
-    GPtrArray *installations = flatpak_get_system_installations(m_cancellable, error);
+    g_autoptr(GPtrArray) installations = flatpak_get_system_installations(m_cancellable, error);
     if (*error) {
         qWarning() << "Failed to call flatpak_get_system_installations:" << (*error)->message;
     }
     for (uint i = 0; installations && i < installations->len; i++) {
-        m_installations << FLATPAK_INSTALLATION(g_ptr_array_index(installations, i));
+        auto installation = FLATPAK_INSTALLATION(g_ptr_array_index(installations, i));
+        g_object_ref(installation);
+        m_installations << installation;
     }
 
     auto user = flatpak_installation_new_user(m_cancellable, error);
@@ -954,7 +953,7 @@ void FlatpakBackend::updateAppInstalledMetadata(FlatpakInstalledRef *installedRe
         resource->setState(AbstractResource::Installed);
 }
 
-bool FlatpakBackend::updateAppMetadata(FlatpakInstallation* flatpakInstallation, FlatpakResource *resource)
+bool FlatpakBackend::updateAppMetadata(FlatpakResource *resource)
 {
     if (resource->resourceType() != FlatpakResource::DesktopApp) {
         return true;
@@ -966,27 +965,27 @@ bool FlatpakBackend::updateAppMetadata(FlatpakInstallation* flatpakInstallation,
         return updateAppMetadata(resource, path);
     } else {
         auto fw = new QFutureWatcher<QByteArray>(this);
-        connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, flatpakInstallation, resource, fw]() {
+        connect(fw, &QFutureWatcher<QByteArray>::finished, this, [this, resource, fw]() {
             const auto metadata = fw->result();
             if (!metadata.isEmpty())
-                onFetchMetadataFinished(flatpakInstallation, resource, metadata);
+                onFetchMetadataFinished(resource, metadata);
             fw->deleteLater();
         });
-        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, flatpakInstallation, resource));
+        fw->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchMetadata, resource, m_cancellable));
 
         // Return false to indicate we cannot continue (right now used only in updateAppSize())
         return false;
     }
 }
 
-void FlatpakBackend::onFetchMetadataFinished(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource, const QByteArray &metadata)
+void FlatpakBackend::onFetchMetadataFinished(FlatpakResource *resource, const QByteArray &metadata)
 {
     updateAppMetadata(resource, metadata);
 
     // Right now we attempt to update metadata for calculating the size so call updateSizeFromRemote()
     // as it's what we want. In future if there are other reason to update metadata we will need to somehow
     // distinguish between these calls
-    updateAppSizeFromRemote(flatpakInstallation, resource);
+    updateAppSizeFromRemote(resource);
 }
 
 bool FlatpakBackend::updateAppMetadata(FlatpakResource *resource, const QString &path)
@@ -1013,7 +1012,7 @@ bool FlatpakBackend::updateAppMetadata(FlatpakResource *resource, const QByteArr
     return true;
 }
 
-bool FlatpakBackend::updateAppSize(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource)
+bool FlatpakBackend::updateAppSize(FlatpakResource *resource)
 {
     // Check if the size is already set, we should also distinguish between download and installed size,
     // right now it doesn't matter whether we get size for installed or not installed app, but if we
@@ -1032,25 +1031,25 @@ bool FlatpakBackend::updateAppSize(FlatpakInstallation *flatpakInstallation, Fla
 
     // Check if we know the needed runtime which is needed for calculating the size
     if (resource->runtime().isEmpty()) {
-        if (!updateAppMetadata(flatpakInstallation, resource)) {
+        if (!updateAppMetadata(resource)) {
             return false;
         }
     }
 
-    return updateAppSizeFromRemote(flatpakInstallation, resource);
+    return updateAppSizeFromRemote(resource);
 }
 
-bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource)
+bool FlatpakBackend::updateAppSizeFromRemote(FlatpakResource *resource)
 {
     // Calculate the runtime size
     if (resource->state() == AbstractResource::None && resource->resourceType() == FlatpakResource::DesktopApp) {
         auto runtime = getRuntimeForApp(resource);
         if (runtime) {
             // Re-check runtime state if case a new one was created
-            updateAppState(flatpakInstallation, runtime);
+            updateAppState(runtime);
 
             if (!runtime->isInstalled()) {
-                if (!updateAppSize(flatpakInstallation, runtime)) {
+                if (!updateAppSize(runtime)) {
                     qWarning() << "Failed to get runtime size needed for total size of" << resource->name();
                     return false;
                 }
@@ -1063,7 +1062,7 @@ bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstall
 
     if (resource->state() == AbstractResource::Installed) {
         g_autoptr(FlatpakInstalledRef) ref = nullptr;
-        ref = getInstalledRefForApp(flatpakInstallation, resource);
+        ref = getInstalledRefForApp(resource);
         if (!ref) {
             qWarning() << "Failed to get installed size of" << resource->name();
             return false;
@@ -1075,18 +1074,25 @@ bool FlatpakBackend::updateAppSizeFromRemote(FlatpakInstallation *flatpakInstall
             return false;
         }
 
-        auto futureWatcher = new QFutureWatcher<FlatpakRunnables::SizeInformation>(this);
-        connect(futureWatcher, &QFutureWatcher<FlatpakRunnables::SizeInformation>::finished, this, [this, resource, futureWatcher]() {
-            auto value = futureWatcher->result();
-            if (value.valid) {
-                onFetchSizeFinished(resource, value.downloadSize, value.installedSize);
+        if (resource->propertyState(FlatpakResource::DownloadSize) == FlatpakResource::Fetching) {
+            return true;
+        }
+
+        auto futureWatcher = new QFutureWatcher<FlatpakRemoteRef*>(this);
+        connect(futureWatcher, &QFutureWatcher<FlatpakRemoteRef*>::finished, this, [this, resource, futureWatcher]() {
+            g_autoptr(FlatpakRemoteRef) remoteRef = futureWatcher->result();
+            if (remoteRef) {
+                onFetchSizeFinished(resource, flatpak_remote_ref_get_download_size(remoteRef), flatpak_remote_ref_get_installed_size(remoteRef));
             } else {
                 resource->setPropertyState(FlatpakResource::DownloadSize, FlatpakResource::UnknownOrFailed);
                 resource->setPropertyState(FlatpakResource::InstalledSize, FlatpakResource::UnknownOrFailed);
             }
             futureWatcher->deleteLater();
         });
-        futureWatcher->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::fetchFlatpakSize, flatpakInstallation, resource));
+        resource->setPropertyState(FlatpakResource::DownloadSize, FlatpakResource::Fetching);
+        resource->setPropertyState(FlatpakResource::InstalledSize, FlatpakResource::Fetching);
+
+        futureWatcher->setFuture(QtConcurrent::run(&m_threadPool, &FlatpakRunnables::findRemoteRef, resource, m_cancellable));
     }
 
     return true;
@@ -1107,9 +1113,9 @@ void FlatpakBackend::onFetchSizeFinished(FlatpakResource *resource, guint64 down
     resource->setInstalledSize(installedSize);
 }
 
-void FlatpakBackend::updateAppState(FlatpakInstallation *flatpakInstallation, FlatpakResource *resource)
+void FlatpakBackend::updateAppState(FlatpakResource *resource)
 {
-    FlatpakInstalledRef *ref = getInstalledRefForApp(flatpakInstallation, resource);
+    g_autoptr(FlatpakInstalledRef) ref = getInstalledRefForApp(resource);
     if (ref) {
         // If the app is installed, we can set information about commit, arch etc.
         updateAppInstalledMetadata(ref, resource);
@@ -1203,9 +1209,19 @@ ResultsStream * FlatpakBackend::search(const AbstractResourcesBackend::Filters &
 QVector<AbstractResource *> FlatpakBackend::resourcesByAppstreamName(const QString& name) const
 {
     QVector<AbstractResource*> resources;
-    foreach(FlatpakResource* res, m_resources) {
-        if (QString::compare(res->appstreamId(), name, Qt::CaseInsensitive)==0 || QString::compare(res->appstreamId(), name + QLatin1String(".desktop"), Qt::CaseInsensitive)==0)
+    const QString nameWithDesktop = name + QLatin1String(".desktop");
+    for (FlatpakResource* res : m_resources) {
+        if (QString::compare(res->appstreamId(), name, Qt::CaseInsensitive)==0 || QString::compare(res->appstreamId(), nameWithDesktop, Qt::CaseInsensitive)==0)
             resources << res;
+        else {
+            const auto alts = res->alternativeAppstreamIds();
+            for (const auto &alt : alts) {
+                if (QString::compare(alt, name, Qt::CaseInsensitive)==0 || QString::compare(alt, nameWithDesktop, Qt::CaseInsensitive)==0) {
+                    resources << res;
+                    break;
+                }
+            }
+        }
     }
     auto f = [this](AbstractResource* l, AbstractResource* r) { return flatpakResourceLessThan(l, r); };
     std::sort(resources.begin(), resources.end(), f);
@@ -1221,7 +1237,7 @@ ResultsStream * FlatpakBackend::findResourceByPackageName(const QUrl &url)
         else {
             auto stream = new ResultsStream(QStringLiteral("FlatpakStream"));
             auto f = [this, stream, appstreamIds] () {
-                const auto resources = kAppend<QVector<AbstractResource*>>(appstreamIds, [this] (const QString appstreamId) { return resourcesByAppstreamName(appstreamId); });
+                const auto resources = kAppend<QVector<AbstractResource*>>(appstreamIds, [this] (const QString &appstreamId) { return resourcesByAppstreamName(appstreamId); });
                 if (!resources.isEmpty())
                     Q_EMIT stream->resourcesFound(resources);
                 stream->finish();
@@ -1270,8 +1286,7 @@ Transaction* FlatpakBackend::installApplication(AbstractResource *app, const Add
     FlatpakJobTransaction *transaction = new FlatpakJobTransaction(resource, Transaction::InstallRole);
     connect(transaction, &FlatpakJobTransaction::statusChanged, this, [this, resource] (Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
-            FlatpakInstallation *installation = resource->installation();
-            updateAppState(installation, resource);
+            updateAppState(resource);
         }
     });
     return transaction;
@@ -1294,12 +1309,11 @@ Transaction* FlatpakBackend::removeApplication(AbstractResource *app)
         return nullptr;
     }
 
-    FlatpakInstallation *installation = resource->installation();
     FlatpakJobTransaction *transaction = new FlatpakJobTransaction(resource, Transaction::RemoveRole);
 
-    connect(transaction, &FlatpakJobTransaction::statusChanged, this, [this, installation, resource] (Transaction::Status status) {
+    connect(transaction, &FlatpakJobTransaction::statusChanged, this, [this, resource] (Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
-            updateAppSize(installation, resource);
+            updateAppSize(resource);
         }
     });
     return transaction;
