@@ -1,23 +1,10 @@
-/***************************************************************************
- *   Copyright © 2014 Aleix Pol Gonzalez <aleixpol@blue-systems.com>       *
- *   Copyright © 2017 Jan Grulich <jgrulich@redhat.com>                    *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- ***************************************************************************/
+/*
+ *   SPDX-FileCopyrightText: 2014 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *   SPDX-FileCopyrightText: 2017 Jan Grulich <jgrulich@redhat.com>
+ *
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+ */
+
 
 #include "FlatpakSourcesBackend.h"
 #include "FlatpakResource.h"
@@ -53,7 +40,8 @@ FlatpakSourcesBackend::FlatpakSourcesBackend(const QVector<FlatpakInstallation *
     , m_flathubAction(new QAction(i18n("Add Flathub"), this))
     , m_noSourcesItem(new QStandardItem(QStringLiteral("-")))
 {
-    m_flathubAction->setToolTip(QStringLiteral("flathub"));
+    m_flathubAction->setObjectName(QStringLiteral("flathub"));
+    m_flathubAction->setToolTip(i18n("Makes it possible to easily install the applications listed in https://flathub.org"));
     connect(m_flathubAction, &QAction::triggered, this, [this](){
         addSource(QStringLiteral("https://flathub.org/repo/flathub.flatpakrepo"));
     });
@@ -80,6 +68,9 @@ FlatpakSourcesBackend::~FlatpakSourcesBackend()
     auto conf = KSharedConfig::openConfig();
     KConfigGroup group = conf->group("FlatpakSources");
     group.writeEntry("Sources", ids);
+
+    if (!m_noSourcesItem->model())
+        delete m_noSourcesItem;
 }
 
 QAbstractItemModel* FlatpakSourcesBackend::sources()
@@ -154,59 +145,63 @@ bool FlatpakSourcesBackend::removeSource(const QString &id)
         const auto installation = sourceItem->flatpakInstallation();
 
         g_autoptr(GPtrArray) refs = flatpak_installation_list_remote_refs_sync(installation, id.toUtf8().constData(), cancellable, &error);
-        QHash<QString, QStringList> toRemoveHash;
-        toRemoveHash.reserve(refs->len);
-        QStringList toRemoveRefs;
-        toRemoveRefs.reserve(refs->len);
-        FlatpakBackend* backend = qobject_cast<FlatpakBackend*>(parent());
-        for (uint i = 0; i < refs->len; i++) {
-            FlatpakRef *ref= FLATPAK_REF(g_ptr_array_index(refs, i));
+        if (refs) {
+            QHash<QString, QStringList> toRemoveHash;
+            toRemoveHash.reserve(refs->len);
+            QStringList toRemoveRefs;
+            toRemoveRefs.reserve(refs->len);
+            FlatpakBackend* backend = qobject_cast<FlatpakBackend*>(parent());
+            for (uint i = 0; i < refs->len; i++) {
+                FlatpakRef *ref= FLATPAK_REF(g_ptr_array_index(refs, i));
 
-            g_autoptr(GError) error = nullptr;
-            FlatpakInstalledRef* installedRef = flatpak_installation_get_installed_ref(installation, flatpak_ref_get_kind(ref), flatpak_ref_get_name(ref), flatpak_ref_get_arch(ref), flatpak_ref_get_branch(ref), cancellable, &error);
-            if (installedRef) {
-                auto res = backend->getAppForInstalledRef(installation, installedRef);
-                const auto name = QString::fromUtf8(flatpak_ref_get_name(ref));
-                const auto refString = QString::fromUtf8(flatpak_ref_format_ref(ref));
-                if (!name.endsWith(QLatin1String(".Locale"))) {
-                    if (res)
-                        toRemoveHash[res->name()] << refString;
-                    else
-                        toRemoveHash[refString] << refString;
+                g_autoptr(GError) error = nullptr;
+                FlatpakInstalledRef* installedRef = flatpak_installation_get_installed_ref(installation, flatpak_ref_get_kind(ref), flatpak_ref_get_name(ref), flatpak_ref_get_arch(ref), flatpak_ref_get_branch(ref), cancellable, &error);
+                if (installedRef) {
+                    auto res = backend->getAppForInstalledRef(installation, installedRef);
+                    const auto name = QString::fromUtf8(flatpak_ref_get_name(ref));
+                    const auto refString = QString::fromUtf8(flatpak_ref_format_ref(ref));
+                    if (!name.endsWith(QLatin1String(".Locale"))) {
+                        if (res)
+                            toRemoveHash[res->name()] << refString;
+                        else
+                            toRemoveHash[refString] << refString;
+                    }
+                    toRemoveRefs << refString;
                 }
-                toRemoveRefs << refString;
             }
-        }
-        QStringList toRemove;
-        toRemove.reserve(toRemoveHash.count());
-        for (auto it = toRemoveHash.constBegin(), itEnd = toRemoveHash.constEnd(); it != itEnd; ++it) {
-            if (it.value().count() > 1)
-                toRemove << QStringLiteral("%1 - %2").arg(it.key(), it.value().join(QLatin1String(", ")));
-            else
-                toRemove << it.key();
-        }
-        toRemove.sort();
+            QStringList toRemove;
+            toRemove.reserve(toRemoveHash.count());
+            for (auto it = toRemoveHash.constBegin(), itEnd = toRemoveHash.constEnd(); it != itEnd; ++it) {
+                if (it.value().count() > 1)
+                    toRemove << QStringLiteral("%1 - %2").arg(it.key(), it.value().join(QLatin1String(", ")));
+                else
+                    toRemove << it.key();
+            }
+            toRemove.sort();
 
-        if (!toRemove.isEmpty()) {
-            m_proceedFunctions.push([this, toRemoveRefs, installation, id] {
-                g_autoptr(GError) localError = nullptr;
-                g_autoptr(GCancellable) cancellable = g_cancellable_new();
-                g_autoptr(FlatpakTransaction) transaction = flatpak_transaction_new_for_installation(installation, cancellable, &localError);
-                for (const QString& instRef : qAsConst(toRemoveRefs)) {
-                    const QByteArray refString = instRef.toUtf8();
-                    flatpak_transaction_add_uninstall(transaction, refString.constData(), &localError);
-                    if (localError)
-                        return;
-                }
+            if (!toRemove.isEmpty()) {
+                m_proceedFunctions.push([this, toRemoveRefs, installation, id] {
+                    g_autoptr(GError) localError = nullptr;
+                    g_autoptr(GCancellable) cancellable = g_cancellable_new();
+                    g_autoptr(FlatpakTransaction) transaction = flatpak_transaction_new_for_installation(installation, cancellable, &localError);
+                    for (const QString& instRef : qAsConst(toRemoveRefs)) {
+                        const QByteArray refString = instRef.toUtf8();
+                        flatpak_transaction_add_uninstall(transaction, refString.constData(), &localError);
+                        if (localError)
+                            return;
+                    }
 
-                if (flatpak_transaction_run(transaction, cancellable, &localError)) {
-                    removeSource(id);
-                }
-            });
+                    if (flatpak_transaction_run(transaction, cancellable, &localError)) {
+                        removeSource(id);
+                    }
+                });
 
 
-            Q_EMIT proceedRequest(i18n("Removing '%1'", id), i18n("To remove this repository, the following applications must be uninstalled:<ul><li>%1</li></ul>", toRemove.join(QStringLiteral("</li><li>"))));
-            return false;
+                Q_EMIT proceedRequest(i18n("Removing '%1'", id), i18n("To remove this repository, the following applications must be uninstalled:<ul><li>%1</li></ul>", toRemove.join(QStringLiteral("</li><li>"))));
+                return false;
+            }
+        } else {
+            qWarning() << "could not list refs in repo" << id << error->message;
         }
 
         if (flatpak_installation_remove_remote(installation, id.toUtf8().constData(), cancellable, &error)) {
@@ -308,7 +303,7 @@ void FlatpakSourcesBackend::addRemote(FlatpakRemote *remote, FlatpakInstallation
     const auto theActions = actions();
     for(const QVariant& act: theActions) {
         QAction* action = qobject_cast<QAction*>(act.value<QObject*>());
-        if (action->toolTip() == id) {
+        if (action->objectName() == id) {
             action->setEnabled(false);
             action->setVisible(false);
         }
@@ -356,7 +351,7 @@ void FlatpakSourcesBackend::addRemote(FlatpakRemote *remote, FlatpakInstallation
 
 QString FlatpakSourcesBackend::idDescription()
 {
-    return i18n("Flatpak repository URI (*.flatpakrepo)");
+    return i18n("Enter a Flatpak repository URI (*.flatpakrepo):");
 }
 
 bool FlatpakSourcesBackend::moveSource(const QString& sourceId, int delta)
