@@ -1,23 +1,9 @@
-/***************************************************************************
- *   Copyright © 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>       *
- *   Copyright © 2018 Abhijeet Sharma <sharma.abhijeet2096@gmail.com>      *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- ***************************************************************************/
+/*
+ *   SPDX-FileCopyrightText: 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *   SPDX-FileCopyrightText: 2018 Abhijeet Sharma <sharma.abhijeet2096@gmail.com>
+ *
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+ */
 
 #include "FwupdResource.h"
 
@@ -27,9 +13,15 @@
 #include <QStringList>
 #include <QTimer>
 
-FwupdResource::FwupdResource(QString name, AbstractResourcesBackend* parent)
+FwupdResource::FwupdResource(FwupdDevice* device, AbstractResourcesBackend* parent)
+    : FwupdResource(device, QStringLiteral("org.fwupd.%1.device").arg(QString::fromUtf8(fwupd_device_get_id(device)).replace(QLatin1Char('/'),QLatin1Char('_'))), parent)
+{}
+
+FwupdResource::FwupdResource(FwupdDevice* device, const QString &id, AbstractResourcesBackend* parent)
     : AbstractResource(parent)
-    , m_name(std::move(name))
+    , m_id(id)
+    , m_name(QString::fromUtf8(fwupd_device_get_name(device)))
+    , m_deviceID(QString::fromUtf8(fwupd_device_get_id(device)))
 {
     Q_ASSERT(!m_name.isEmpty());
     setObjectName(m_name);
@@ -135,12 +127,10 @@ void FwupdResource::fetchChangelog()
 
 void FwupdResource::setState(AbstractResource::State state)
 {
-    if(m_state != state)
-    {
+    if(m_state != state) {
         m_state = state;
         emit stateChanged();
     }
-
 }
 
 void FwupdResource::invokeApplication() const
