@@ -15,7 +15,6 @@
 #include <QDebug>
 #include <QTimer>
 #include <KLocalizedString>
-#include <PackageKit/Transaction>
 #include <PackageKit/Daemon>
 #include <functional>
 
@@ -96,6 +95,9 @@ void PKTransaction::trigger(PackageKit::Transaction::TransactionFlags flags)
     connect(m_trans.data(), &PackageKit::Transaction::statusChanged, this, &PKTransaction::statusChanged);
     connect(m_trans.data(), &PackageKit::Transaction::eulaRequired, this, &PKTransaction::eulaRequired);
     connect(m_trans.data(), &PackageKit::Transaction::allowCancelChanged, this, &PKTransaction::cancellableChanged);
+    connect(m_trans.data(), &PackageKit::Transaction::remainingTimeChanged, this, [this]() {
+        setRemainingTime(m_trans->remainingTime());
+    });
     connect(m_trans.data(), &PackageKit::Transaction::speedChanged, this, [this]() {
         setDownloadSpeed(m_trans->speed());
     });
@@ -214,7 +216,11 @@ void PKTransaction::proceed()
     if (!m_proceedFunctions.isEmpty()) {
         processProceedFunction();
     } else {
-        trigger(PackageKit::Transaction::TransactionFlagOnlyTrusted);
+        if (m_apps.size() == 1 && qobject_cast<LocalFilePKResource*>(m_apps.at(0))) {
+            trigger(PackageKit::Transaction::TransactionFlagNone);
+        } else {
+            trigger(PackageKit::Transaction::TransactionFlagOnlyTrusted);
+        }
     }
 }
 
