@@ -7,7 +7,6 @@
 
 #include "PackageKitBackend.h"
 #include "PackageKitSourcesBackend.h"
-#include "PackageKitResource.h"
 #include "PackageKitUpdater.h"
 #include "AppPackageKitResource.h"
 #include "PKTransaction.h"
@@ -23,7 +22,6 @@
 #include <QProcess>
 #include <QStringList>
 #include <QDebug>
-#include <QTimer>
 #include <QStandardPaths>
 #include <QFile>
 #include <QAction>
@@ -32,7 +30,6 @@
 #include <QFutureWatcher>
 #include <QtConcurrentRun>
 
-#include <PackageKit/Transaction>
 #include <PackageKit/Daemon>
 #include <PackageKit/Offline>
 #include <PackageKit/Details>
@@ -106,6 +103,8 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
             fetchUpdates();
         acquireFetching(false);
     }, this);
+
+    PackageKit::Daemon::global()->setHints(QStringLiteral("locale=%1").arg(qEnvironmentVariable("LANG")));
 }
 
 PackageKitBackend::~PackageKitBackend()
@@ -656,6 +655,10 @@ Transaction* PackageKitBackend::installApplication(AbstractResource* app)
 Transaction* PackageKitBackend::removeApplication(AbstractResource* app)
 {
     Q_ASSERT(!isFetching());
+    if (!qobject_cast<PackageKitResource*>(app)) {
+        Q_EMIT passiveMessage(i18n("Cannot remove '%1'", app->name()));
+        return nullptr;
+    }
     return new PKTransaction({app}, Transaction::RemoveRole);
 }
 
