@@ -1,32 +1,18 @@
-/***************************************************************************
- *   Copyright © 2012 Aleix Pol Gonzalez <aleixpol@blue-systems.com>       *
- *   Copyright © 2013 Lukas Appelhans <l.appelhans@gmx.de>                 *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- ***************************************************************************/
+/*
+ *   SPDX-FileCopyrightText: 2012 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *   SPDX-FileCopyrightText: 2013 Lukas Appelhans <l.appelhans@gmx.de>
+ *
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+ */
 
 #include "PackageKitResource.h"
 #include "PackageKitBackend.h"
 #include "PackageKitMessages.h"
 #include <KLocalizedString>
-#include <PackageKit/Details>
 #include <PackageKit/Daemon>
 #include <QJsonArray>
 #include <QDebug>
+#include <utils.h>
 
 #if defined(WITH_MARKDOWN)
 extern "C" {
@@ -167,6 +153,8 @@ void PackageKitResource::addPackageId(PackageKit::Transaction::Info info, const
 
     if (oldState != state())
         emit stateChanged();
+
+    Q_EMIT versionsChanged();
 }
 
 QStringList PackageKitResource::categories()
@@ -209,11 +197,19 @@ void PackageKitResource::setDetails(const PackageKit::Details & details)
         return;
 
     if (m_details != details) {
+        const auto oldState = state();
+        const auto oldSize= size();
         m_details = details;
-        emit stateChanged();
+
+        if (oldState != state())
+            emit stateChanged();
 
         if (!backend()->isFetching())
             Q_EMIT backend()->resourcesChanged(this, {"size", "homepage", "license"});
+
+        if (oldSize != size()) {
+            Q_EMIT sizeChanged();
+        }
     }
 }
 
@@ -225,7 +221,8 @@ void PackageKitResource::fetchUpdateDetails()
 {
     const auto pkgid = availablePackageId();
     if (pkgid.isEmpty()) {
-        connect(this, &PackageKitResource::stateChanged, this, &PackageKitResource::fetchUpdateDetails);
+        auto a = new OneTimeAction([this] { fetchUpdateDetails(); }, this);
+        connect(this, &PackageKitResource::stateChanged, a, &OneTimeAction::trigger);
         return;
     }
     PackageKit::Transaction* t = PackageKit::Daemon::getUpdateDetail(availablePackageId());
@@ -266,8 +263,8 @@ void PackageKitResource::updateDetail(const QString& packageID, const QStringLis
                                       const QString& /*changelog*/, PackageKit::Transaction::UpdateState state, const QDateTime& /*issued*/, const QDateTime& /*updated*/)
 {
 #if defined(WITH_MARKDOWN)
-    const char* xx = _updateText.toUtf8().constData();
-    MMIOT *markdownHandle = mkd_string(xx, _updateText.size(), 0);
+    const QByteArray xx = _updateText.toUtf8();
+    MMIOT *markdownHandle = mkd_string(xx.constData(), _updateText.size(), 0);
 
     QString updateText;
     if ( !mkd_compile( markdownHandle, MKD_FENCEDCODE | MKD_GITHUBTAGS | MKD_AUTOLINK ) ) {
