@@ -1,22 +1,8 @@
-/***************************************************************************
- *   Copyright © 2013 Lukas Appelhans <l.appelhans@gmx.de>                 *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- ***************************************************************************/
+/*
+ *   SPDX-FileCopyrightText: 2013 Lukas Appelhans <l.appelhans@gmx.de>
+ *
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+ */
 #include "PackageKitUpdater.h"
 #include "PackageKitMessages.h"
 
@@ -26,10 +12,11 @@
 #include <QAction>
 #include <QSet>
 
-#include <KSharedConfig>
-#include <KConfigGroup>
 #include <KLocalizedString>
 
+#include "libdiscover_backend_debug.h"
+#include "utils.h"
+
 int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage)
 {
     const auto was = percentage;
@@ -43,15 +30,127 @@ int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage
         };
         const auto idx = statuses.value(status, -1);
         if (idx < 0) {
-            qDebug() << "Status not present" << status << "among" << statuses   .keys() << percentage;
+            qCDebug(LIBDISCOVER_BACKEND_LOG) << "Status not present" << status << "among" << statuses   .keys() << percentage;
             return -1;
         }
         percentage = (idx * 100 + percentage) / 2 /*the maximum in statuses*/;
     }
-    qDebug() << "reporting progress with status:" << status << percentage << was;
+    qCDebug(LIBDISCOVER_BACKEND_LOG) << "reporting progress with status:" << status << percentage << was;
     return percentage;
 }
 
+static void kRemoveDuplicates(QJsonArray & input, std::function<QString(const QJsonValueRef &)> fetchKey)
+{
+    QSet<QString> ret;
+    for (auto it = input.begin(); it != input.end(); ) {
+        const auto key = fetchKey(*it);
+        if (!ret.contains(key)) {
+            ret << key;
+            ++it;
+        } else {
+            it = input.erase(it);
+        }
+    }
+}
+
+class SystemUpgrade : public AbstractResource
+{
+public:
+    SystemUpgrade(PackageKitBackend* backend)
+        : AbstractResource(backend)
+        , m_backend(backend)
+    {
+        connect(m_backend, &AbstractResourcesBackend::resourceRemoved, this, [this] (AbstractResource *res) {
+            m_resources.remove(res);
+        });
+    }
+
+    QString packageName() const override { return QStringLiteral("discover-offline-upgrade");}
+    QString name() const override { return i18n("System upgrade"); }
+    QString comment() override { return upgradeText(); }
+    QVariant icon() const override { return QStringLiteral("system-upgrade"); }
+    bool canExecute() const override { return false; }
+    void invokeApplication() const override {}
+    State state() override { return Upgradeable; }
+    QStringList categories() override { return {}; }
+    AbstractResource::Type type() const override { return Technical; }
+    int size() override {
+        int ret = 0;
+        QSet<QString> donePkgs;
+        for (auto res : qAsConst(m_resources)) {
+            PackageKitResource * app = qobject_cast<PackageKitResource*>(res);
+            QString pkgid = m_backend->upgradeablePackageId(app);
+            if (!donePkgs.contains(pkgid)) {
+                donePkgs.insert(pkgid);
+                ret += app->size();
+            }
+        }
+        return ret;
+    }
+    QJsonArray licenses() override {
+        QJsonArray ret;
+        for (auto res : qAsConst(m_resources)) {
+            ret += res->licenses();
+        }
+        kRemoveDuplicates(ret, [] (const QJsonValueRef &val) -> QString { return val.toObject()[QLatin1String("name")].toString(); });
+        return ret;
+    }
+    QString section() override { return {}; }
+    QString longDescription() override { return {}; }
+    QString origin() const override { return {}; }
+    QString author() const override { return {}; }
+    QList<PackageState> addonsInformation() override { return {}; }
+    QString upgradeText() const override {
+        return i18np("1 package to upgrade", "%1 packages to upgrade", m_resources.count());
+    }
+    void fetchChangelog() override {
+        QString changes;
+        for (auto res : qAsConst(m_resources)) {
+            const auto versions = res->upgradeText();
+            const auto idx = versions.indexOf(u'\u009C');
+            changes += QStringLiteral("<li>") + res->packageName() + QStringLiteral(": ") + versions.leftRef(idx) + QStringLiteral("</li>\n");
+        }
+        Q_EMIT changelogFetched(QStringLiteral("<ul>") + changes + QStringLiteral("</ul>\n"));
+    }
+
+    QString installedVersion() const override { return i18n("Present"); }
+    QString availableVersion() const override { return i18n("Future"); }
+    QString sourceIcon() const override { return QStringLiteral("package-x-generic"); }
+    QDate releaseDate() const override { return {}; }
+
+    QSet<AbstractResource*> resources() const {
+        return m_resources;
+    }
+
+    QSet<QString> allPackageNames() const {
+        QSet<QString> ret;
+        for (auto res : qAsConst(m_resources)) {
+            ret += kToSet(qobject_cast<PackageKitResource*>(res)->allPackageNames());
+        }
+        return ret;
+    }
+
+    void refreshResource() {
+        Q_EMIT m_backend->resourcesChanged(this, {"size", "license"});
+    }
+
+    void setCandidates(const QSet<AbstractResource*> &candidates) {
+        for (auto res : (m_resources - candidates)) {
+            disconnect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::refreshResource);
+        }
+
+        const auto newCandidates = (candidates - m_resources);
+        m_resources = candidates;
+        for (auto res : newCandidates) {
+            connect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::refreshResource);
+        }
+    }
+
+private:
+    QSet<AbstractResource*> m_resources;
+    PackageKitBackend* const m_backend;
+};
+
 PackageKitUpdater::PackageKitUpdater(PackageKitBackend * parent)
   : AbstractBackendUpdater(parent),
     m_transaction(nullptr),
@@ -59,7 +158,8 @@ PackageKitUpdater::PackageKitUpdater(PackageKitBackend * parent)
     m_isCancelable(false),
     m_isProgressing(false),
     m_percentage(0),
-    m_lastUpdate()
+    m_lastUpdate(),
+    m_upgrade(new SystemUpgrade(m_backend))
 {
     fetchLastUpdateTime();
 }
@@ -78,14 +178,21 @@ void PackageKitUpdater::prepare()
     }
 
     Q_ASSERT(!m_transaction);
-    m_toUpgrade = m_backend->upgradeablePackages();
+    const auto candidates = m_backend->upgradeablePackages();
+    if (useOfflineUpdates() && !candidates.isEmpty()) {
+        m_upgrade->setCandidates(candidates);
+
+        m_toUpgrade = { m_upgrade };
+    } else {
+        m_toUpgrade = candidates;
+    }
     m_allUpgradeable = m_toUpgrade;
 }
 
 void PackageKitUpdater::setupTransaction(PackageKit::Transaction::TransactionFlags flags)
 {
     m_packagesModified.clear();
-    auto pkgs = involvedPackages(m_toUpgrade).toList();
+    auto pkgs = involvedPackages(m_toUpgrade).values();
     pkgs.sort();
     m_transaction = PackageKit::Daemon::updatePackages(pkgs, flags);
     m_isCancelable = m_transaction->allowCancel();
@@ -95,7 +202,6 @@ void PackageKitUpdater::setupTransaction(PackageKit::Transaction::TransactionFla
     connect(m_transaction.data(), &PackageKit::Transaction::package, this, &PackageKitUpdater::packageResolved);
     connect(m_transaction.data(), &PackageKit::Transaction::errorCode, this, &PackageKitUpdater::errorFound);
     connect(m_transaction.data(), &PackageKit::Transaction::mediaChangeRequired, this, &PackageKitUpdater::mediaChange);
-    connect(m_transaction.data(), &PackageKit::Transaction::requireRestart, this, &PackageKitUpdater::requireRestart);
     connect(m_transaction.data(), &PackageKit::Transaction::eulaRequired, this, &PackageKitUpdater::eulaRequired);
     connect(m_transaction.data(), &PackageKit::Transaction::repoSignatureRequired, this, &PackageKitUpdater::repoSignatureRequired);
     connect(m_transaction.data(), &PackageKit::Transaction::allowCancelChanged, this, &PackageKitUpdater::cancellableChanged);
@@ -108,16 +214,21 @@ void PackageKitUpdater::setupTransaction(PackageKit::Transaction::TransactionFla
 
 QSet<AbstractResource*> PackageKitUpdater::packagesForPackageId(const QSet<QString>& pkgids) const
 {
-    QSet<QString> packages;
-    packages.reserve(pkgids.size());
-    foreach(const QString& pkgid, pkgids) {
-        packages += PackageKit::Daemon::packageName(pkgid);
-    }
+    const auto packages = kTransform<QSet<QString>>(pkgids,
+                                                    [] (const QString& pkgid) { return PackageKit::Daemon::packageName(pkgid); }
+                                                   );
 
     QSet<AbstractResource*> ret;
     foreach (AbstractResource * res, m_allUpgradeable) {
+        if (auto upgrade = dynamic_cast<SystemUpgrade*>(res)) {
+            if (packages.contains(upgrade->allPackageNames())) {
+                ret += upgrade;
+            }
+            continue;
+        }
+
         PackageKitResource* pres = qobject_cast<PackageKitResource*>(res);
-        if (packages.contains(pres->allPackageNames().toSet())) {
+        if (packages.contains(kToSet(pres->allPackageNames()))) {
             ret.insert(res);
         }
     }
@@ -130,9 +241,13 @@ QSet<QString> PackageKitUpdater::involvedPackages(const QSet<AbstractResource*>&
     QSet<QString> packageIds;
     packageIds.reserve(packages.size());
     foreach (AbstractResource * res, packages) {
-        PackageKitResource * app = qobject_cast<PackageKitResource*>(res);
-        QString pkgid = m_backend->upgradeablePackageId(app);
+        if (SystemUpgrade* upgrade = dynamic_cast<SystemUpgrade*>(res)) {
+            packageIds = involvedPackages(upgrade->resources());
+            continue;
+        }
 
+        PackageKitResource * app = qobject_cast<PackageKitResource*>(res);
+        const QString pkgid = m_backend->upgradeablePackageId(app);
         if (pkgid.isEmpty()) {
             qWarning() << "no upgradeablePackageId for" << app;
             continue;
@@ -173,19 +288,12 @@ void PackageKitUpdater::proceed()
 
 bool PackageKitUpdater::useOfflineUpdates() const
 {
-    if (qEnvironmentVariableIsSet("PK_OFFLINE_UPDATE"))
-        return true;
-    KConfigGroup group(KSharedConfig::openConfig(), "Software");
-    return group.readEntry<bool>("UseOfflineUpdates", false);
+    return m_useOfflineUpdates || qEnvironmentVariableIntValue("PK_OFFLINE_UPDATE");
 }
 
-void PackageKitUpdater::setUseOfflineUpdates(bool use)
+void PackageKitUpdater::setOfflineUpdates(bool use)
 {
-//     To enable from command line use:
-//     kwriteconfig5 --file discoverrc --group Software --key UseOfflineUpdates true
-
-    KConfigGroup group(KSharedConfig::openConfig(), "Software");
-    group.writeEntry<bool>("UseOfflineUpdates", use);
+    m_useOfflineUpdates = use;
 }
 
 void PackageKitUpdater::start()
@@ -198,7 +306,7 @@ void PackageKitUpdater::start()
 
 void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time*/)
 {
-//     qDebug() << "update finished!" << exit << time;
+//     qCDebug(LIBDISCOVER_BACKEND_LOG) << "update finished!" << exit << time;
     if (!m_proceedFunctions.isEmpty())
         return;
     const bool cancel = exit == PackageKit::Transaction::ExitCancelled;
@@ -211,7 +319,7 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
         const auto toremove = m_packagesModified.value(PackageKit::Transaction::InfoRemoving);
         if (!toremove.isEmpty()) {
             const auto toinstall = QStringList() << m_packagesModified.value(PackageKit::Transaction::InfoInstalling) << m_packagesModified.value(PackageKit::Transaction::InfoUpdating);
-            Q_EMIT proceedRequest(i18n("Packages to remove"), i18n("The following packages will be removed by the update:\n<ul><li>%1</li></ul>\nin order to install:\n<ul><li>%2</li></ul>",
+            Q_EMIT proceedRequest(i18n("Packages to remove"), i18n("The following packages will be removed by the update:<ul><li>%1</li></ul><br/>in order to install:<ul><li>%2</li></ul>",
                                                                    PackageKitResource::joinPackages(toremove, QStringLiteral("</li><li>"), {}),
                                                                    PackageKitResource::joinPackages(toinstall, QStringLiteral("</li><li>"), {})
                                                                   ));
@@ -227,7 +335,6 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
 
     if (useOfflineUpdates() && exit == PackageKit::Transaction::ExitSuccess) {
         PackageKit::Daemon::global()->offline()->trigger(PackageKit::Offline::ActionReboot);
-        Q_EMIT passiveMessage(i18n("Please restart the computer to finish the update"));
     }
 }
 
@@ -260,19 +367,19 @@ qreal PackageKitUpdater::progress() const
 
 void PackageKitUpdater::removeResources(const QList<AbstractResource*>& apps)
 {
-    QSet<QString> pkgs = involvedPackages(apps.toSet());
+    const QSet<QString> pkgs = involvedPackages(kToSet(apps));
     m_toUpgrade.subtract(packagesForPackageId(pkgs));
 }
 
 void PackageKitUpdater::addResources(const QList<AbstractResource*>& apps)
 {
-    QSet<QString> pkgs = involvedPackages(apps.toSet());
+    const QSet<QString> pkgs = involvedPackages(kToSet(apps));
     m_toUpgrade.unite(packagesForPackageId(pkgs));
 }
 
 QList<AbstractResource*> PackageKitUpdater::toUpdate() const
 {
-    return m_toUpgrade.toList();
+    return m_toUpgrade.values();
 }
 
 bool PackageKitUpdater::isMarked(AbstractResource* res) const
@@ -317,11 +424,6 @@ void PackageKitUpdater::mediaChange(PackageKit::Transaction::MediaType media, co
     Q_EMIT passiveMessage(i18n("Media Change of type '%1' is requested.\n%2", type, text));
 }
 
-void PackageKitUpdater::requireRestart(PackageKit::Transaction::Restart restart, const QString& pkgid)
-{
-    Q_EMIT passiveMessage(PackageKitMessages::restartMessage(restart, pkgid));
-}
-
 void PackageKitUpdater::eulaRequired(const QString& eulaID, const QString& packageID, const QString& vendor, const QString& licenseAgreement)
 {
     m_proceedFunctions << [eulaID](){
@@ -373,7 +475,7 @@ AbstractBackendUpdater::State toUpdateState(PackageKit::Transaction::Status t)
         case PackageKit::Transaction::StatusCancel:
             return AbstractBackendUpdater::Done;
         default:
-            qDebug() << "unknown packagekit status" << t;
+            qCDebug(LIBDISCOVER_BACKEND_LOG) << "unknown packagekit status" << t;
             return AbstractBackendUpdater::None;
     }
     Q_UNREACHABLE();
@@ -392,7 +494,11 @@ void PackageKitUpdater::fetchChangelog() const
 {
     QStringList pkgids;
     foreach(AbstractResource* res, m_allUpgradeable) {
-        pkgids += static_cast<PackageKitResource*>(res)->availablePackageId();
+        if (auto upgrade = dynamic_cast<SystemUpgrade*>(res)) {
+            upgrade->fetchChangelog();
+        } else {
+            pkgids += static_cast<PackageKitResource*>(res)->availablePackageId();
+        }
     }
     Q_ASSERT(!pkgids.isEmpty());
 
@@ -435,6 +541,11 @@ double PackageKitUpdater::updateSize() const
     double ret = 0.;
     QSet<QString> donePkgs;
     for (AbstractResource * res : m_toUpgrade) {
+        if (auto upgrade = dynamic_cast<SystemUpgrade*>(res)) {
+            ret += upgrade->size();
+            continue;
+        }
+
         PackageKitResource * app = qobject_cast<PackageKitResource*>(res);
         QString pkgid = m_backend->upgradeablePackageId(app);
         if (!donePkgs.contains(pkgid)) {
