@@ -1,22 +1,8 @@
-/***************************************************************************
- *   Copyright © 2012 Aleix Pol Gonzalez <aleixpol@blue-systems.com>       *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- ***************************************************************************/
+/*
+ *   SPDX-FileCopyrightText: 2012 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+ */
 
 #include "ResourcesModel.h"
 
@@ -70,7 +56,7 @@ ResourcesModel::ResourcesModel(QObject* parent, bool load)
                 return 0;
 
             int sum = 0;
-            for(auto backend: m_backends) {
+            for(auto backend: qAsConst(m_backends)) {
                 sum += backend->fetchingUpdatesProgress();
             }
             return sum / m_backends.count();
@@ -101,7 +87,7 @@ void ResourcesModel::init(bool load)
     m_updateAction = new QAction(this);
     m_updateAction->setIcon(QIcon::fromTheme(QStringLiteral("system-software-update")));
     m_updateAction->setText(i18nc("@action Checks the Internet for updates", "Check for Updates"));
-    m_updateAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_R));
+    m_updateAction->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_R));
     connect(this, &ResourcesModel::fetchingChanged, m_updateAction, [this](bool fetching) {
         m_updateAction->setEnabled(!fetching);
         m_fetchingUpdatesProgress.reevaluate();
@@ -148,8 +134,10 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend* backend)
     connect(backend, &AbstractResourcesBackend::fetchingUpdatesProgressChanged, this, [this] { m_fetchingUpdatesProgress.reevaluate(); });
     connect(backend, &AbstractResourcesBackend::resourceRemoved, this, &ResourcesModel::resourceRemoved);
     connect(backend, &AbstractResourcesBackend::passiveMessage, this, &ResourcesModel::passiveMessage);
-    connect(backend->reviewsBackend(), &AbstractReviewsBackend::error, this, &ResourcesModel::passiveMessage, Qt::UniqueConnection);
     connect(backend->backendUpdater(), &AbstractBackendUpdater::progressingChanged, this, &ResourcesModel::slotFetching);
+    if (backend->reviewsBackend()) {
+        connect(backend->reviewsBackend(), &AbstractReviewsBackend::error, this, &ResourcesModel::passiveMessage, Qt::UniqueConnection);
+    }
 
     // In case this is in fact the first backend to be added, and also happens to be
     // pre-filled, we still need for the rest of the backends to be added before trying
@@ -304,7 +292,7 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream*>& str
 
     for (auto stream: streams) {
         connect(stream, &ResultsStream::resourcesFound, this, &AggregatedResultsStream::addResults);
-        connect(stream, &QObject::destroyed, this, &AggregatedResultsStream::destruction);
+        connect(stream, &QObject::destroyed, this, &AggregatedResultsStream::streamDestruction);
         connect(this, &ResultsStream::fetchMore, stream, &ResultsStream::fetchMore);
         m_streams << stream;
     }
@@ -318,9 +306,7 @@ AggregatedResultsStream::~AggregatedResultsStream() = default;
 void AggregatedResultsStream::addResults(const QVector<AbstractResource *>& res)
 {
     for(auto r : res)
-        connect(r, &QObject::destroyed, this, [this, r](){
-            m_results.removeAll(r);
-        });
+        connect(r, &QObject::destroyed, this, &AggregatedResultsStream::resourceDestruction);
 
     m_results += res;
 
@@ -337,7 +323,12 @@ void AggregatedResultsStream::emitResults()
     m_delayedEmission.stop();
 }
 
-void AggregatedResultsStream::destruction(QObject* obj)
+void AggregatedResultsStream::resourceDestruction(QObject* obj)
+{
+    m_results.removeAll(qobject_cast<AbstractResource*>(obj));
+}
+
+void AggregatedResultsStream::streamDestruction(QObject* obj)
 {
     m_streams.remove(obj);
     clear();
@@ -368,14 +359,9 @@ void ResourcesModel::checkForUpdates()
         backend->checkForUpdates();
 }
 
-QVector<AbstractResourcesBackend *> ResourcesModel::applicationBackends() const
+QVariantList ResourcesModel::backendsVariant() const
 {
-    return kFilter<QVector<AbstractResourcesBackend*>>(m_backends, [](AbstractResourcesBackend* b){ return b->hasApplications(); });
-}
-
-QVariantList ResourcesModel::applicationBackendsVariant() const
-{
-    return kTransform<QVariantList>(applicationBackends(), [](AbstractResourcesBackend* b) {return QVariant::fromValue<QObject*>(b);});
+    return kTransform<QVariantList>(m_backends, [](AbstractResourcesBackend* b) {return QVariant::fromValue<QObject*>(b);});
 }
 
 AbstractResourcesBackend* ResourcesModel::currentApplicationBackend() const
@@ -404,13 +390,12 @@ void ResourcesModel::initApplicationsBackend()
 {
     const auto name = applicationSourceName();
 
-    const auto backends = applicationBackends();
-    auto idx = kIndexOf(backends, [name](AbstractResourcesBackend* b) { return b->name() == name; });
+    auto idx = kIndexOf(m_backends, [name](AbstractResourcesBackend* b) { return b->hasApplications() && b->name() == name; });
     if (idx<0) {
-        idx = kIndexOf(backends, [](AbstractResourcesBackend* b) { return b->hasApplications(); });
+        idx = kIndexOf(m_backends, [](AbstractResourcesBackend* b) { return b->hasApplications(); });
         qCDebug(LIBDISCOVER_LOG) << "falling back applications backend to" << idx;
     }
-    setCurrentApplicationBackend(backends.value(idx, nullptr), false);
+    setCurrentApplicationBackend(m_backends.value(idx, nullptr), false);
 }
 
 QString ResourcesModel::applicationSourceName() const
